package org.gcube.portlets.user.td.expressionwidget.server;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.gcube.data.analysis.tabulardata.expression.Expression;
import org.gcube.data.analysis.tabulardata.expression.MultivaluedExpression;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.Avg;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.Count;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.Max;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.Min;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.ST_Extent;
import org.gcube.data.analysis.tabulardata.expression.composite.aggregation.Sum;
import org.gcube.data.analysis.tabulardata.expression.composite.arithmetic.Addition;
import org.gcube.data.analysis.tabulardata.expression.composite.arithmetic.Division;
import org.gcube.data.analysis.tabulardata.expression.composite.arithmetic.Modulus;
import org.gcube.data.analysis.tabulardata.expression.composite.arithmetic.Multiplication;
import org.gcube.data.analysis.tabulardata.expression.composite.arithmetic.Subtraction;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.Equals;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.GreaterOrEquals;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.GreaterThan;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.LessOrEquals;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.LessThan;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.NotEquals;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.NotGreater;
import org.gcube.data.analysis.tabulardata.expression.composite.comparable.NotLess;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Concat;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Levenshtein;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Lower;
import org.gcube.data.analysis.tabulardata.expression.composite.text.MD5;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Similarity;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Soundex;
import org.gcube.data.analysis.tabulardata.expression.composite.text.SubstringByIndex;
import org.gcube.data.analysis.tabulardata.expression.composite.text.SubstringByRegex;
import org.gcube.data.analysis.tabulardata.expression.composite.text.SubstringPosition;
import org.gcube.data.analysis.tabulardata.expression.composite.text.TextBeginsWith;
import org.gcube.data.analysis.tabulardata.expression.composite.text.TextContains;
import org.gcube.data.analysis.tabulardata.expression.composite.text.TextEndsWith;
import org.gcube.data.analysis.tabulardata.expression.composite.text.TextMatchSQLRegexp;
import org.gcube.data.analysis.tabulardata.expression.composite.text.TextReplaceMatchingRegex;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Trim;
import org.gcube.data.analysis.tabulardata.expression.composite.text.Upper;
import org.gcube.data.analysis.tabulardata.expression.functions.Cast;
import org.gcube.data.analysis.tabulardata.expression.leaf.ColumnReferencePlaceholder;
import org.gcube.data.analysis.tabulardata.expression.leaf.ConstantList;
import org.gcube.data.analysis.tabulardata.expression.leaf.Range;
import org.gcube.data.analysis.tabulardata.expression.logical.And;
import org.gcube.data.analysis.tabulardata.expression.logical.Between;
import org.gcube.data.analysis.tabulardata.expression.logical.IsNotNull;
import org.gcube.data.analysis.tabulardata.expression.logical.IsNull;
import org.gcube.data.analysis.tabulardata.expression.logical.Not;
import org.gcube.data.analysis.tabulardata.expression.logical.Or;
import org.gcube.data.analysis.tabulardata.expression.logical.ValueIsIn;
import org.gcube.data.analysis.tabulardata.model.column.ColumnLocalId;
import org.gcube.data.analysis.tabulardata.model.column.ColumnReference;
import org.gcube.data.analysis.tabulardata.model.datatype.BooleanType;
import org.gcube.data.analysis.tabulardata.model.datatype.DataType;
import org.gcube.data.analysis.tabulardata.model.datatype.DateType;
import org.gcube.data.analysis.tabulardata.model.datatype.GeometryType;
import org.gcube.data.analysis.tabulardata.model.datatype.IntegerType;
import org.gcube.data.analysis.tabulardata.model.datatype.NumericType;
import org.gcube.data.analysis.tabulardata.model.datatype.TextType;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDBoolean;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDDate;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDGeometry;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDInteger;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDNumeric;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDText;
import org.gcube.data.analysis.tabulardata.model.datatype.value.TDTypeValue;
import org.gcube.data.analysis.tabulardata.model.table.TableId;
import org.gcube.portlets.user.td.expressionwidget.shared.exception.ExpressionParserException;
import org.gcube.portlets.user.td.expressionwidget.shared.expression.C_MultivaluedExpression;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_Avg;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_Count;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_Max;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_Min;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_ST_Extent;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.aggregation.C_Sum;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.arithmetic.C_Addition;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.arithmetic.C_Division;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.arithmetic.C_Modulus;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.arithmetic.C_Multiplication;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.arithmetic.C_Subtraction;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_Equals;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_GreaterOrEquals;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_GreaterThan;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_LessOrEquals;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_LessThan;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_NotEquals;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_NotGreater;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.comparable.C_NotLess;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.functions.C_Cast;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Concat;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Levenshtein;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Lower;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_MD5;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Similarity;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Soundex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringByIndex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringByRegex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringPosition;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextBeginsWith;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextContains;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextEndsWith;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextMatchSQLRegexp;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextReplaceMatchingRegex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Trim;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Upper;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_ColumnReference;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_ColumnReferencePlaceholder;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_ConstantList;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_Range;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.TD_Value;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_And;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_Between;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_IsNotNull;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_IsNull;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_Not;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_Or;
import org.gcube.portlets.user.td.expressionwidget.shared.model.logical.C_ValueIsIn;
import org.gcube.portlets.user.td.widgetcommonevent.shared.expression.C_Expression;
import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnDataType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class C_ExpressionParser {
	protected static Logger logger = LoggerFactory
			.getLogger(C_ExpressionParser.class);

	protected SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

	public Expression parse(C_Expression exp) throws ExpressionParserException {
		logger.debug("Parse: " + exp);
		Expression ex = null;

		switch (exp.getId()) {
		case "ColumnReferencePlaceholder":
			ex = getColumnReferencePlaceholder(exp);
			break;
		case "ConstantList":
			ex = getConstantlist(exp);
			break;
		case "Range":
			ex = getRange(exp);
			break;
		case "ColumnReference":
			ex = getColumnReference(exp);
			break;
		case "TD_Value":
			TD_Value value = (TD_Value) exp;
			ex = getExpressionValue(value);
			break;
		case "Equals":
			ex = getEquals(exp);
			break;
		case "GreaterOrEquals":
			ex = getGreaterOrEquals(exp);
			break;
		case "GreaterThan":
			ex = getGreaterThan(exp);
			break;
		case "LessOrEquals":
			ex = getLessOrEquals(exp);
			break;
		case "LessThan":
			ex = getLessThan(exp);
			break;
		case "NotEquals":
			ex = getNotEquals(exp);
			break;
		case "NotGreater":
			ex = getNotGreater(exp);
			break;
		case "NotLess":
			ex = getNotLess(exp);
			break;
		case "TextBeginsWith":
			ex = getTextBeginWith(exp);
			break;
		case "TextContains":
			ex = getTextContains(exp);
			break;
		case "TextEndsWith":
			ex = getTextEndWith(exp);
			break;
		case "TextMatchSQLRegexp":
			ex = getTextMatchSQLRegexp(exp);
			break;
		case "And":
			ex = getAnd(exp);
			break;
		case "Between":
			ex = getBetween(exp);
			break;
		case "IsNotNull":
			ex = getIsNotNull(exp);
			break;
		case "IsNull":
			ex = getIsNull(exp);
			break;
		case "Not":
			ex = getNot(exp);
			break;
		case "Or":
			ex = getOr(exp);
			break;
		case "ValueIsIn":
			ex = getValueIsIn(exp);
			break;
		case "Concat":
			ex = getConcat(exp);
			break;
		case "SubstringByIndex":
			ex = getSubstringByIndex(exp);
			break;
		case "SubstringByRegex":
			ex = getSubstringByRegex(exp);
			break;
		case "SubstringPosition":
			ex = getSubstringPosition(exp);
			break;
		case "TextReplaceMatchingRegex":
			ex = getTextReplaceMatchingRegex(exp);
			break;
		case "Avg":
			ex = getAvg(exp);
			break;
		case "Count":
			ex = getCount(exp);
			break;
		case "Max":
			ex = getMax(exp);
			break;
		case "Min":
			ex = getMin(exp);
			break;
		case "Sum":
			ex = getSum(exp);
			break;
		case "ST_Extent":
			ex = getSTExtent(exp);
			break;
		case "Cast":
			ex = getCast(exp);
			break;
		case "Addition":
			ex = getAddition(exp);
			break;
		case "Subtraction":
			ex = getSubtraction(exp);
			break;
		case "Modulus":
			ex = getModulus(exp);
			break;
		case "Multiplication":
			ex = getMultiplication(exp);
			break;
		case "Division":
			ex = getDivision(exp);
			break;
		case "Upper":
			ex = getUpper(exp);
			break;
		case "Lower":
			ex = getLower(exp);
			break;
		case "Trim":
			ex = getTrim(exp);
			break;
		case "MD5":
			ex = getMD5(exp);
			break;
		case "Soundex":
			ex = getSoundex(exp);
			break;
		case "Levenshtein":
			ex = getLevenshtein(exp);
			break;
		case "Similarity":
			ex = getSimilarity(exp);
			break;	
		default:	
			break;
		}

		return ex;
	}

	public MultivaluedExpression parseMultivalued(C_MultivaluedExpression exp)
			throws ExpressionParserException {
		MultivaluedExpression ex = null;
		switch (exp.getIdMulti()) {
		case "ConstantList":
			ex = getConstantlist(exp);
			break;
		case "Range":
			ex = getRange(exp);
			break;
		case "ExternalReferenceExpression":
			break;
		default:
			break;
		}
		return ex;
	}

	private Expression getValueIsIn(C_Expression exp)
			throws ExpressionParserException {
		C_ValueIsIn v = (C_ValueIsIn) exp;
		MultivaluedExpression multivalued = parseMultivalued(v
				.getRightArgument());
		ValueIsIn valueIsIn = new ValueIsIn(parse(v.getLeftArgument()),
				multivalued);
		return valueIsIn;
	}

	private Expression getOr(C_Expression exp) throws ExpressionParserException {
		C_Or o = (C_Or) exp;
		List<C_Expression> listCExp = o.getArguments();
		List<Expression> listExp = new ArrayList<Expression>();
		Expression ex;
		for (C_Expression cexp : listCExp) {
			ex = parse(cexp);
			listExp.add(ex);
		}
		Or or = new Or(listExp);
		return or;
	}

	private Expression getNot(C_Expression exp)
			throws ExpressionParserException {
		C_Not n = (C_Not) exp;
		Not not = new Not(parse(n.getArgument()));
		return not;
	}

	private Expression getIsNull(C_Expression exp)
			throws ExpressionParserException {
		C_IsNull nullIs = (C_IsNull) exp;
		IsNull isNull = new IsNull(parse(nullIs.getArgument()));
		return isNull;
	}

	private Expression getIsNotNull(C_Expression exp)
			throws ExpressionParserException {
		C_IsNotNull nullNotIs = (C_IsNotNull) exp;
		IsNotNull isNotNull = new IsNotNull(parse(nullNotIs.getArgument()));
		return isNotNull;
	}

	private Expression getBetween(C_Expression exp)
			throws ExpressionParserException {
		C_Between bet = (C_Between) exp;
		C_Range crange = bet.getRightArgument();
		Range range = new Range(getExpressionValue(crange.getMinimum()),
				getExpressionValue(crange.getMaximum()));
		Between between = new Between(parse(bet.getLeftArgument()), range);

		return between;
	}

	private Expression getAnd(C_Expression exp)
			throws ExpressionParserException {
		C_And o = (C_And) exp;
		List<C_Expression> listCExp = o.getArguments();
		List<Expression> listExp = new ArrayList<Expression>();
		Expression ex;
		for (C_Expression cexp : listCExp) {
			ex = parse(cexp);
			listExp.add(ex);
		}
		And and = new And(listExp);
		return and;
	}

	private Expression getTextMatchSQLRegexp(C_Expression exp)
			throws ExpressionParserException {
		C_TextMatchSQLRegexp reg = (C_TextMatchSQLRegexp) exp;
		TextMatchSQLRegexp regExp = new TextMatchSQLRegexp(
				parse(reg.getLeftArgument()), parse(reg.getRightArgument()));
		return regExp;

	}

	private Expression getTextEndWith(C_Expression exp)
			throws ExpressionParserException {
		C_TextEndsWith textEnd = (C_TextEndsWith) exp;
		TextEndsWith textEndWith = new TextEndsWith(
				parse(textEnd.getLeftArgument()),
				parse(textEnd.getRightArgument()));
		return textEndWith;
	}

	private Expression getTextContains(C_Expression exp)
			throws ExpressionParserException {
		C_TextContains textContains = (C_TextContains) exp;
		TextContains textCont = new TextContains(
				parse(textContains.getLeftArgument()),
				parse(textContains.getRightArgument()));
		return textCont;
	}

	private Expression getTextBeginWith(C_Expression exp)
			throws ExpressionParserException {
		C_TextBeginsWith textB = (C_TextBeginsWith) exp;
		TextBeginsWith textBegins = new TextBeginsWith(
				parse(textB.getLeftArgument()), parse(textB.getRightArgument()));
		return textBegins;
	}

	private Expression getNotLess(C_Expression exp)
			throws ExpressionParserException {
		C_NotLess notL = (C_NotLess) exp;
		NotLess notLess = new NotLess(parse(notL.getLeftArgument()),
				parse(notL.getRightArgument()));
		return notLess;
	}

	private Expression getNotGreater(C_Expression exp)
			throws ExpressionParserException {
		C_NotGreater notG = (C_NotGreater) exp;
		NotGreater notGreater = new NotGreater(parse(notG.getLeftArgument()),
				parse(notG.getRightArgument()));
		return notGreater;
	}

	private Expression getNotEquals(C_Expression exp)
			throws ExpressionParserException {
		C_NotEquals notE = (C_NotEquals) exp;
		NotEquals notEquals = new NotEquals(parse(notE.getLeftArgument()),
				parse(notE.getRightArgument()));
		return notEquals;
	}

	private Expression getLessThan(C_Expression exp)
			throws ExpressionParserException {
		C_LessThan lessT = (C_LessThan) exp;
		LessThan lessThan = new LessThan(parse(lessT.getLeftArgument()),
				parse(lessT.getRightArgument()));
		return lessThan;
	}

	private Expression getLessOrEquals(C_Expression exp)
			throws ExpressionParserException {
		C_LessOrEquals lessOrE = (C_LessOrEquals) exp;
		LessOrEquals lessOrEquals = new LessOrEquals(
				parse(lessOrE.getLeftArgument()),
				parse(lessOrE.getRightArgument()));
		return lessOrEquals;
	}

	private Expression getGreaterThan(C_Expression exp)
			throws ExpressionParserException {
		C_GreaterThan greaterThan = (C_GreaterThan) exp;
		GreaterThan greater = new GreaterThan(
				parse(greaterThan.getLeftArgument()),
				parse(greaterThan.getRightArgument()));
		return greater;

	}

	private Expression getGreaterOrEquals(C_Expression exp)
			throws ExpressionParserException {
		C_GreaterOrEquals greaterOrEq = (C_GreaterOrEquals) exp;
		GreaterOrEquals greaterOrEquals = new GreaterOrEquals(
				parse(greaterOrEq.getLeftArgument()),
				parse(greaterOrEq.getRightArgument()));
		return greaterOrEquals;

	}

	protected TDTypeValue getExpressionValue(TD_Value value)
			throws ExpressionParserException {

		TDTypeValue ex = null;
		try {
			switch (value.getValueType()) {
			case Boolean:
				ex = new TDBoolean(Boolean.valueOf(value.getValue()));
				break;
			case Date:
				Date d = null;
				try {
					d = sdf.parse(value.getValue());
				} catch (ParseException e) {
					logger.error("Unparseable using " + sdf);
					throw new ExpressionParserException(value.getValue()
							+ " is not valid Date type");
				}
				ex = new TDDate(d);
				break;
			case Geometry:
				if (TDGeometry.validateGeometry(value.getValue())) {
					ex = new TDGeometry(value.getValue());
				} else {
					throw new ExpressionParserException(value.getValue()
							+ " is not valid Geometry type");
				}
				break;
			case Integer:
				Integer vInteger;
				try {
					vInteger = Integer.valueOf(value.getValue());
				} catch (NumberFormatException e) {
					throw new ExpressionParserException(value.getValue()
							+ " is not valid Integer type");
				}
				ex = new TDInteger(vInteger);
				break;
			case Numeric:
				Double vNumeric;
				try {
					vNumeric = Double.valueOf(value.getValue());
				} catch (NumberFormatException e) {
					throw new ExpressionParserException(value.getValue()
							+ " is not valid Double type");
				}
				ex = new TDNumeric(vNumeric);
				break;
			case Text:
				if (value.getValue() == null) {
					ex = new TDText("");
				} else {
					ex = new TDText(value.getValue());
				}
				break;
			default:
				break;
			}
		} catch (Throwable e) {
			logger.error("type error parsing value " + value + " "
					+ e.getLocalizedMessage());
		}
		return ex;
	}

	protected DataType mapColumnDataType(ColumnDataType columnDataType) {
		if (columnDataType == ColumnDataType.Integer) {
			return new IntegerType();
		} else {
			if (columnDataType == ColumnDataType.Numeric) {
				return new NumericType();
			} else {
				if (columnDataType == ColumnDataType.Boolean) {
					return new BooleanType();
				} else {
					if (columnDataType == ColumnDataType.Geometry) {
						return new GeometryType();
					} else {
						if (columnDataType == ColumnDataType.Text) {
							return new TextType();
						} else {
							if (columnDataType == ColumnDataType.Date) {
								return new DateType();
							} else {
								return null;
							}
						}
					}
				}
			}
		}
	}

	protected Expression getConstantlist(C_Expression exp)
			throws ExpressionParserException {
		List<TDTypeValue> l = new ArrayList<TDTypeValue>();
		C_ConstantList c = (C_ConstantList) exp;
		List<TD_Value> arguments = c.getArguments();
		TDTypeValue tdv;
		for (TD_Value value : arguments) {
			tdv = getExpressionValue(value);
			l.add(tdv);
		}
		ConstantList constList = new ConstantList(l);
		return constList;
	}

	protected MultivaluedExpression getConstantlist(C_MultivaluedExpression exp)
			throws ExpressionParserException {
		List<TDTypeValue> l = new ArrayList<TDTypeValue>();
		C_ConstantList c = (C_ConstantList) exp;
		List<TD_Value> arguments = c.getArguments();
		TDTypeValue tdv;
		for (TD_Value value : arguments) {
			tdv = getExpressionValue(value);
			l.add(tdv);
		}
		ConstantList constList = new ConstantList(l);
		return constList;
	}

	protected Expression getColumnReferencePlaceholder(C_Expression exp) {
		C_ColumnReferencePlaceholder c = (C_ColumnReferencePlaceholder) exp;
		ColumnReferencePlaceholder col = new ColumnReferencePlaceholder(
				mapColumnDataType(c.getDataType()), c.getColumnId());
		return col;
	}

	protected Expression getRange(C_Expression exp)
			throws ExpressionParserException {
		C_Range c = (C_Range) exp;
		Range range = new Range(getExpressionValue(c.getMinimum()),
				getExpressionValue(c.getMaximum()));
		return range;
	}

	protected MultivaluedExpression getRange(C_MultivaluedExpression exp)
			throws ExpressionParserException {
		C_Range c = (C_Range) exp;
		Range range = new Range(getExpressionValue(c.getMinimum()),
				getExpressionValue(c.getMaximum()));
		return range;
	}

	protected Expression getColumnReference(C_Expression exp) {
		C_ColumnReference c = (C_ColumnReference) exp;
		String tableIdS = null;
		if (c.getTrId().isViewTable()) {
			tableIdS = c.getTrId().getReferenceTargetTableId();
		} else {
			tableIdS = c.getTrId().getTableId();
		}
		TableId tableId = new TableId(Long.valueOf(tableIdS));
		ColumnLocalId columnId = new ColumnLocalId(c.getColumnId());

		ColumnReference ref = new ColumnReference(tableId, columnId,
				mapColumnDataType(c.getDataType()));
		return ref;
	}

	protected Expression getUpper(C_Expression exp)
			throws ExpressionParserException {
		C_Upper e = (C_Upper) exp;
		Expression arg = parse(e.getArgument());
		Upper upper = new Upper(arg);
		return upper;
	}
	
	protected Expression getLower(C_Expression exp)
			throws ExpressionParserException {
		C_Lower e = (C_Lower) exp;
		Expression arg = parse(e.getArgument());
		Lower lower = new Lower(arg);
		return lower;
	}
	
	protected Expression getTrim(C_Expression exp)
			throws ExpressionParserException {
		C_Trim e = (C_Trim) exp;
		Expression arg = parse(e.getArgument());
		Trim trim = new Trim(arg);
		return trim;
	}
	
	protected Expression getMD5(C_Expression exp)
			throws ExpressionParserException {
		C_MD5 e = (C_MD5) exp;
		Expression arg = parse(e.getArgument());
		MD5 md5 = new MD5(arg);
		return md5;
	}

	protected Expression getSoundex(C_Expression exp)
			throws ExpressionParserException {
		C_Soundex e = (C_Soundex) exp;
		Expression arg = parse(e.getArgument());
		Soundex soundex = new Soundex(arg);
		return soundex;
	}
	
	protected Expression getLevenshtein(C_Expression exp)
			throws ExpressionParserException {
		C_Levenshtein e = (C_Levenshtein) exp;
		Expression left = parse(e.getLeftArgument());
		Expression right = parse(e.getRightArgument());
		
		Levenshtein levenshtein = new Levenshtein(left,right);
		return levenshtein;
	}
	
	protected Expression getSimilarity(C_Expression exp)
			throws ExpressionParserException {
		C_Similarity e = (C_Similarity) exp;
		Expression left = parse(e.getLeftArgument());
		Expression right = parse(e.getRightArgument());
		
		Similarity similarity = new Similarity(left,right);
		return similarity;
	}
	
	

	protected Expression getEquals(C_Expression exp)
			throws ExpressionParserException {
		C_Equals e = (C_Equals) exp;
		Expression left = parse(e.getLeftArgument());
		Expression right = parse(e.getRightArgument());
		Equals eq = new Equals(left, right);
		return eq;
	}

	private Expression getConcat(C_Expression exp)
			throws ExpressionParserException {
		C_Concat concat = (C_Concat) exp;
		Concat conc = new Concat(parse(concat.getLeftArgument()),
				parse(concat.getRightArgument()));
		return conc;
	}

	private Expression getSubstringByIndex(C_Expression exp)
			throws ExpressionParserException {
		C_SubstringByIndex subByIndex = (C_SubstringByIndex) exp;
		SubstringByIndex sByIndex = new SubstringByIndex(
				parse(subByIndex.getSourceString()),
				parse(subByIndex.getFromIndex()),
				parse(subByIndex.getToIndex()));
		return sByIndex;
	}

	private Expression getSubstringByRegex(C_Expression exp)
			throws ExpressionParserException {
		C_SubstringByRegex subByRegex = (C_SubstringByRegex) exp;
		SubstringByRegex sByRegex = new SubstringByRegex(
				parse(subByRegex.getSourceString()),
				parse(subByRegex.getRegex()));
		return sByRegex;
	}

	private Expression getSubstringPosition(C_Expression exp)
			throws ExpressionParserException {
		C_SubstringPosition subPosition = (C_SubstringPosition) exp;
		SubstringPosition sPosition = new SubstringPosition(
				parse(subPosition.getLeftArgument()),
				parse(subPosition.getRightArgument()));
		return sPosition;
	}

	private Expression getTextReplaceMatchingRegex(C_Expression exp)
			throws ExpressionParserException {
		C_TextReplaceMatchingRegex textReplaceMatchingRegex = (C_TextReplaceMatchingRegex) exp;
		TDText tdRegexp = new TDText(textReplaceMatchingRegex.getRegexp()
				.getValue());

		String replacement = textReplaceMatchingRegex.getReplacing().getValue();
		if (replacement == null) {
			replacement = "";
		}
		TDText tdReplacing = new TDText(replacement);

		TextReplaceMatchingRegex textRepRegex = new TextReplaceMatchingRegex(
				parse(textReplaceMatchingRegex.getToCheckText()), tdRegexp,
				tdReplacing);
		return textRepRegex;
	}

	private Expression getAvg(C_Expression exp)
			throws ExpressionParserException {
		C_Avg avg = (C_Avg) exp;
		Avg av = new Avg(parse(avg.getArgument()));
		return av;
	}

	private Expression getCount(C_Expression exp)
			throws ExpressionParserException {
		C_Count count = (C_Count) exp;
		Count cnt = new Count(parse(count.getArgument()));
		return cnt;
	}

	private Expression getMax(C_Expression exp)
			throws ExpressionParserException {
		C_Max max = (C_Max) exp;
		Max ma = new Max(parse(max.getArgument()));
		return ma;
	}

	private Expression getMin(C_Expression exp)
			throws ExpressionParserException {
		C_Min min = (C_Min) exp;
		Min mi = new Min(parse(min.getArgument()));
		return mi;
	}

	private Expression getSTExtent(C_Expression exp)
			throws ExpressionParserException {
		C_ST_Extent stExtent = (C_ST_Extent) exp;
		ST_Extent stEx = new ST_Extent(parse(stExtent.getArgument()));
		return stEx;
	}

	private Expression getSum(C_Expression exp)
			throws ExpressionParserException {
		C_Sum sum = (C_Sum) exp;
		Sum sm = new Sum(parse(sum.getArgument()));
		return sm;
	}

	private Expression getCast(C_Expression exp)
			throws ExpressionParserException {
		C_Cast castExp = (C_Cast) exp;
		Cast sm = new Cast(parse(castExp.getLeftArgument()),
				mapColumnDataType(castExp.getRightArgument()));
		return sm;
	}

	private Expression getAddition(C_Expression exp)
			throws ExpressionParserException {
		C_Addition addition = (C_Addition) exp;
		Addition add = new Addition(parse(addition.getLeftArgument()),
				parse(addition.getRightArgument()));
		return add;
	}

	private Expression getSubtraction(C_Expression exp)
			throws ExpressionParserException {
		C_Subtraction subtraction = (C_Subtraction) exp;
		Subtraction sub = new Subtraction(parse(subtraction.getLeftArgument()),
				parse(subtraction.getRightArgument()));
		return sub;
	}

	private Expression getModulus(C_Expression exp)
			throws ExpressionParserException {
		C_Modulus modulus = (C_Modulus) exp;
		Modulus modu = new Modulus(parse(modulus.getLeftArgument()),
				parse(modulus.getRightArgument()));
		return modu;
	}

	private Expression getMultiplication(C_Expression exp)
			throws ExpressionParserException {
		C_Multiplication multiplication = (C_Multiplication) exp;
		Multiplication multi = new Multiplication(
				parse(multiplication.getLeftArgument()),
				parse(multiplication.getRightArgument()));
		return multi;
	}

	private Expression getDivision(C_Expression exp)
			throws ExpressionParserException {
		C_Division division = (C_Division) exp;
		Division divi = new Division(parse(division.getLeftArgument()),
				parse(division.getRightArgument()));
		return divi;
	}

}
